<?php

namespace Drupal\privatemsg\Plugin\Action;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\views_bulk_operations\Action\ViewsBulkOperationsActionBase;

/**
 * Mark thread as unread.
 *
 * @Action(
 *   id = "privatemsg_unread_thread_action",
 *   label = @Translation("Mark as unread"),
 *   type = "privatemsg_thread",
 *   confirm = FALSE,
 *   requirements = {
 *     "_permission" = "privatemsg use messages actions",
 *   },
 * )
 */
class PrivatemsgUnreadThreadAction extends ViewsBulkOperationsActionBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    return $object->access('mark_unread', $account, $return_as_object);
  }

  /**
   * {@inheritdoc}
   */
  public function execute(ContentEntityInterface $entity = NULL) {
    if ($entity) {
      /** @var \Drupal\privatemsg\PrivateMsgServiceInterface $privatemsg_service */
      $privatemsg_service = \Drupal::service('privatemsg.common');
      $privatemsg_service->markThreadGroupAsUnread(\Drupal::currentUser()->id(), $entity->getGroup());
      return 'Marked selected threads as unread';
    }

    return 'You must first select one (or more) messages before you can take that action.';
  }

}
